package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesReworkTaskService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesReworkTaskModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesReworkTaskRequestModel;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesPart;
import cn.estsh.i3plus.pojo.mes.bean.MesPartTypePicture;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import cn.estsh.i3plus.pojo.mes.bean.MesProductionAssembly;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspection;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspectionDetail;
import cn.estsh.i3plus.pojo.mes.bean.rework.MesReworkTask;
import cn.estsh.i3plus.pojo.mes.bean.rework.MesReworkTaskDetail;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.map.HashedMap;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;


/**
 * @Description: 查询返工单
 * @Author: zxw
 * @Date: 2024/5/21 14:16
 * @Modify:
 */
@Service
@Slf4j
public class MesReworkTaskServiceImpl implements IMesReworkTaskService {

    private static final Logger LOGGER = LoggerFactory.getLogger(MesReworkTaskServiceImpl.class);


    @Autowired
    private MesReworkTaskRepository mesReworkTaskRepository;
    @Autowired
    private MesPartTypePictureRepository mesPartTypePictureRepository;

    @Autowired
    private MesPartRepository mesPartRepository;

    @Autowired
    private MesPartInspectionRepository mesPartInspectionRepository;

    @Autowired
    private MesPartInspectionDetailRepository mesPartInspectionDetailRepository;

    @Autowired
    private MesReworkTaskDetailRepository mesReworkTaskDetailRepository;

    @Autowired
    private MesProductionAssemblyRepository mesProductionAssemblyRepository;

    @Autowired
    private MesProduceSnRepository mesProduceSnRepository;

    @Override
    public ListPager<MesReworkTask> queryReworkTask(MesReworkTask mesReworkTask, Pager pager) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(AuthUtil.getOrganizeCode());
        if (mesReworkTask.getSn() != null) {
            DdlPreparedPack.getStringEqualPack(mesReworkTask.getSn(), "sn", packBean);
        }
        DdlPreparedPack.getStringEqualPack(mesReworkTask.getReworkOrder(), "reworkOrder", packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.REWORK_TASK_STATUS.CREATE.getValue(), "status", packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);

        pager = PagerHelper.getPager(pager, mesReworkTaskRepository.findByHqlWhereCount(packBean));

        List<MesReworkTask> resultList = mesReworkTaskRepository.findByHqlWherePage(packBean, pager);

        return new ListPager<>(resultList, pager);
    }


    @Override
    public MesReworkTaskModel queryReworkTaskModel(MesReworkTaskRequestModel requestModel){
        /**
         * 获取返工单信息
         */
        MesReworkTask res = getMesReworkTask(requestModel);

        /**
         * 获取返工单信息
         */
        List<MesReworkTaskDetail> reworkTaskDetails = getMesReworkTaskDetail(res);
        /**
         * 获取零件图片信息
         */
        MesPartTypePicture mesPartTypePicture = getMesPartTypePicture(res);
        /**
         * 获取缺陷记录
         */
        MesPartInspection mesPartInspection = getMesPartInspection(res);

        List<MesPartInspectionDetail> mesPartInspectionDetails = null;
        if (!CollectionUtils.isEmpty(reworkTaskDetails)) {
            /**
             * 获取零件缺陷明细列表
             */
            mesPartInspectionDetails = getMesPartInspectionDetail(reworkTaskDetails);
        }
        /**
         * 组装返回信息
         */
        MesReworkTaskModel mesReworkTaskModel = MesReworkTaskModel.builder()
                .reworkOrder(res.getReworkOrder())
                .sn(res.getSn())
                .mesPartTypePicture(mesPartTypePicture)
                .mesPartInspection(mesPartInspection)
                .mesPartInspectionDetails(mesPartInspectionDetails).build();


        return mesReworkTaskModel;
    }

    @Override
    public void reworkRecord(long detailId) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(AuthUtil.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(detailId, "partInspectionDetailId", packBean);

        List<MesReworkTaskDetail> mesReworkTaskDetailList = mesReworkTaskDetailRepository.findByHqlWhere(packBean);
        if (CollectionUtils.isEmpty(mesReworkTaskDetailList)) {
            MesPcnException.throwMesBusiException("返工单详情不存在");
        }
        mesReworkTaskDetailList.forEach(mesReworkTaskDetail->{
            mesReworkTaskDetail.setStatus(MesExtEnumUtil.REWORK_TASK_DETAIL_STATUS.REWORK_TASK_DETAIL_STATUS_20.getValue());
            ConvertBean.serviceModelUpdate(mesReworkTaskDetail,AuthUtil.getSessionUser().getUserName());
        });
        mesReworkTaskDetailRepository.saveAll(mesReworkTaskDetailList);

    }


    @Override
    public List<MesProductionAssembly> assemblyQuery(MesReworkTaskRequestModel requestModel) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(requestModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(requestModel.getCustSn(), "custSn", packBean);
        DdlPreparedPack.getNumEqualPack(requestModel.getIsOrigSn(), "isOrigSn", packBean);

        //DdlPreparedPack.getStringEqualPack(requestModel.getPartNo(), "partNo", packBean);
        if (!StringUtils.isEmpty(requestModel.getAssemblyPartNo())) {
            DdlPreparedPack.getStringEqualPack(requestModel.getAssemblyPartNo(), "assemblyPartNo", packBean);
        }
        if (!Objects.isNull(requestModel.getAssemblyPartStatus())) {
            DdlPreparedPack.getNumEqualPack(requestModel.getAssemblyPartStatus(), "assemblyStatus", packBean);
        }

        List<MesProductionAssembly> mesProductionAssemblies = mesProductionAssemblyRepository.findByHqlWhere(packBean);
        if (CollectionUtils.isEmpty(mesProductionAssemblies)) {
            MesPcnException.throwFlowException("装配件信息不存在");
        }
        return mesProductionAssemblies;
    }

    @Override
    public void reworkSuccess(MesReworkTaskRequestModel requestModel) {

        MesReworkTask mesReworkTask = getMesReworkTask(requestModel);
        /**
         * 获取返工单信息
         */
        List<MesReworkTaskDetail> reworkTaskDetails = getMesReworkTaskDetail(mesReworkTask);
        if (!CollectionUtils.isEmpty(reworkTaskDetails)) {
            MesPcnException.throwFlowException("还有位置未返工完成,请检查");
        }
        if (MesExtEnumUtil.PART_INSPECTION_SOURCE_TYPE.LOT.getValue() == mesReworkTask.getType()) {
            mesReworkTask.setReworkedQty(requestModel.getReworkedQty());
            mesReworkTask.setScrapQty(requestModel.getScrapQty());
        } else {
            if (MesExtEnumUtil.NC_TYPE.REWORK.getValue() == requestModel.getType()) {
                mesReworkTask.setReworkedQty(mesReworkTask.getQty());
            } else if (MesExtEnumUtil.NC_TYPE.SCRAP.getValue() == requestModel.getType()) {
                //更新条码状态为报废
                updateMesProduceSn(requestModel);
                //更新返工数量
                mesReworkTask.setScrapQty(mesReworkTask.getQty());
            }
        }

        mesReworkTask.setStatus(MesExtEnumUtil.REWORK_TASK_STATUS.FINISH.getValue());
        mesReworkTaskRepository.save(mesReworkTask);
    }

    @Override
    public boolean validateSn(MesReworkTaskRequestModel requestModel) {
        //
        List<String> snList = Arrays.asList(requestModel.getSn().split(","));
        List<String> idList = Arrays.asList(requestModel.getAssemblyIds().split(","));
        // 判断数量是否一致
        if (snList.size() != idList.size()) MesPcnException.throwFlowException("选中的装配件个数和输入的个数不一致");
        // 判断顺序是否一致
        Map<String, String> map = IntStream.range(0, snList.size()).boxed().collect(Collectors.toMap(idList::get, snList::get, (a, b) -> b));
        List<Long> idLonglist = idList.stream().map(str -> Long.parseLong(str)).collect(Collectors.toList());

        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean();
        DdlPreparedPack.getInPackList(idLonglist, "id", ddlPackBean);
        List<MesProductionAssembly> mesProductionAssemblies = mesProductionAssemblyRepository.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(mesProductionAssemblies)) {
            MesPcnException.throwFlowException("装配件记录不存在");
        }
        Map<Long, String> mapProduction = mesProductionAssemblies.stream().collect(Collectors.toMap(mesProductionAssembly -> mesProductionAssembly.getId(), mesProductionAssembly -> mesProductionAssembly.getPartNo()));
        map.forEach((k, v) -> {
            MesProductionAssembly mesProductionAssembly = mesProductionAssemblyRepository.getById(Long.parseLong(k));
            if (!Objects.equals(mapProduction.get(k), mesProductionAssembly.getPartNo())) {
                MesPcnException.throwFlowException(String.format("装配件记录不匹配id=【%s】,sn=【%s】", k, v));
            }
        });

        return false;
    }

    /**
     * map id和sn对应关系
     * mapProductionAssemble  id 和物料对应关系
     * mapProduceSn 物料和条码对应关系
     *
     * 根据mapProductionAssemble id找map的sn
     * 根据mapProductionAssemble 物料找mmapProduceSn的sn
     * 两者必须一致才允许
     *
     * @param requestModel
     */
    @Override
    public void assemblySnRepeat(MesReworkTaskRequestModel requestModel) {

        try {
            //
            List<String> snList = Arrays.asList(requestModel.getSn().split(","));
            snList = snList.stream().distinct().collect(Collectors.toList());
            List<String> idList = Arrays.asList(requestModel.getAssemblyIds().split(","));
            // 判断数量是否一致
            if (snList.size() != idList.size()) MesPcnException.throwFlowException("选中的装配件个数和输入的个数不一致");
            // 判断顺序是否一致  key = id, value = sn
            Map<String, String> map = IntStream.range(0, snList.size()).boxed().collect(Collectors.toMap(idList::get, snList::get, (a, b) -> b));
            List<Long> idLonglist = idList.stream().map(str -> Long.parseLong(str)).collect(Collectors.toList());

            // 获取装配件记录列表
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean();
            DdlPreparedPack.getInPackList(idLonglist, "id", ddlPackBean);
            List<MesProductionAssembly> mesProductionAssemblies = mesProductionAssemblyRepository.findByHqlWhere(ddlPackBean);
            if (CollectionUtils.isEmpty(mesProductionAssemblies)) {
                MesPcnException.throwFlowException("装配件记录不存在");
            }
            Map<Long, String> mapProductionAssemble = mesProductionAssemblies.stream().collect(Collectors.toMap(mesProductionAssembly -> mesProductionAssembly.getId(), mesProductionAssembly -> mesProductionAssembly.getAssemblyPartNo()));

            List<Integer> snStatusList = Stream.of(MesExtEnumUtil.PRODUCE_SN_STATUS.OFFLINE.getValue(), MesExtEnumUtil.PRODUCE_SN_STATUS.ASSEMBLY_BACK.getValue()).collect(Collectors.toList());
            // 获取替换条码信息列表
            DdlPackBean snDdlPackBean = DdlPackBean.getDdlPackBean();
            DdlPreparedPack.getInPackList(snList, "productSn", snDdlPackBean);
            DdlPreparedPack.getInPackList(snStatusList, "snStatus", snDdlPackBean);

            List<MesProduceSn> mesProduceSnList = mesProduceSnRepository.findByHqlWhere(snDdlPackBean);
            if (CollectionUtils.isEmpty(mesProductionAssemblies)) {
                MesPcnException.throwFlowException("零件条码不存在");
            }
            List<MesProduceSn> mesProduceSns =  new ArrayList<>();
            Map<String, List<MesProduceSn>>  productSnListMap = mesProduceSnList.stream().collect(Collectors.groupingBy(mesProduceSn -> mesProduceSn.getProductSn()));
            Map<String, MesProduceSn> productSnMap = new HashedMap();
            productSnListMap.forEach((k,v) -> {
                MesProduceSn produceSn = v.stream().sorted(Comparator.comparing(MesProduceSn::getCreateDatetime).reversed()).findFirst().get();
                mesProduceSns.add(produceSn);
                productSnMap.put(produceSn.getProductSn(), produceSn);
            });

            Map<String, String> mapProduceSn = mesProduceSns.stream().collect(Collectors.toMap(mesProduceSn -> mesProduceSn.getPartNo(), mesProduceSn -> mesProduceSn.getProductSn()));

            mapProductionAssemble.forEach((k, v) -> {
                String sn = map.get(k + "");
                String newSn = mapProduceSn.get(v);
                if (!Objects.equals(sn, newSn)) {
                    MesPcnException.throwFlowException(String.format("装配件记录不匹配id=【%s】,sn=【%s】", k, sn));
                }
            });

            for (String sn : snList) {
                MesProduceSn mesProduceSn = productSnMap.get(sn);
                if (mesProduceSn == null) {
                    MesPcnException.throwFlowException("条码信息不存在");
                }
                requestModel.setAssemblyPartNo(mesProduceSn.getPartNo());
                requestModel.setAssemblyPartStatus(MesExtEnumUtil.ASSEMBLY_STATUS.ASSEMBLY_STATUS_10.getValue());
                List<MesProductionAssembly> mesProductionAssemblieList = assemblyQuery(requestModel);
                if (CollectionUtils.isEmpty(mesProductionAssemblieList)) {
                    MesPcnException.throwFlowException("未匹配到该条码的零件号");
                }
                // 需要变更之前的记录
                MesProductionAssembly mesProductionAssembly = mesProductionAssemblieList.get(0);
                mesProductionAssembly.setAssemblyStatus(MesExtEnumUtil.ASSEMBLY_STATUS.ASSEMBLY_STATUS_20.getValue());


                // 需要新增一条替换条码的记录
                MesProductionAssembly mesProductionRepeatAssembly = new MesProductionAssembly();
                BeanUtils.copyProperties(mesProductionAssembly, mesProductionRepeatAssembly);
                mesProductionRepeatAssembly.setAssemblySn(sn);
                mesProductionRepeatAssembly.setAssemblyStatus(MesExtEnumUtil.ASSEMBLY_STATUS.ASSEMBLY_STATUS_10.getValue());
                mesProductionRepeatAssembly.setId(null);
                mesProductionRepeatAssembly.setIsOrigSn(CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
                mesProductionRepeatAssembly.setFid(UUID .randomUUID().toString());

                mesProductionAssembly.setRepeatAssemblySn(sn);
                // 更新新替换得条码为已装配
                updateNewSn(mesProduceSn);
                // 更新原条码状态为已拆解
                updateOldSn(mesProductionAssembly);
                // 原装配件记录为已解绑
                mesProductionAssemblyRepository.update(mesProductionAssembly);
                // 新装配件记录为已装配
                mesProductionAssemblyRepository.save(mesProductionRepeatAssembly);
            }
            /**
             * 需要更新NC标记后进入的表明细状态为已处理
             */
            for (String detailId : requestModel.getDetailIds().split(",")) {
                reworkRecord(Long.valueOf(detailId));
            }
        } catch (Exception e) {
            log.error("替换条码异常", e);
            throw e;
        }

    }

    private void updateNewSn(MesProduceSn mesProduceSn) {
        mesProduceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.ASSEMBLY.getValue());
        ConvertBean.serviceModelUpdate(mesProduceSn, mesProduceSn.getCreateUser());
        mesProduceSnRepository.update(mesProduceSn);
    }

    private void updateOldSn(MesProductionAssembly mesProductionAssembly) {
        DdlPackBean oldSnPackBean = DdlPackBean.getDdlPackBean();
        DdlPreparedPack.getStringEqualPack(mesProductionAssembly.getProductSn(), "productSn", oldSnPackBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, oldSnPackBean);

        MesProduceSn oldProduceSn = mesProduceSnRepository.getByProperty(oldSnPackBean);
        if (oldProduceSn == null) {
            MesPcnException.throwFlowException(String.format("原条码信息不存在【%s】", mesProductionAssembly.getProductSn()));
        }

        ConvertBean.serviceModelUpdate(oldProduceSn, oldProduceSn.getCreateUser());
        oldProduceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.ASSEMBLY_BACK.getValue());
        mesProduceSnRepository.update(oldProduceSn);
    }


    private MesPartInspection getMesPartInspection(MesReworkTask mesReworkTask) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(mesReworkTask.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesReworkTask.getSn(), "sn", packBean);

        MesPartInspection mesPartInspection = mesPartInspectionRepository.getByProperty(packBean);

        return mesPartInspection;
    }

    private List<MesPartInspectionDetail> getMesPartInspectionDetail(List<MesReworkTaskDetail> reworkTaskDetails) {

        List<Long> ids = reworkTaskDetails.stream().map(MesReworkTaskDetail::getPartInspectionDetailId).collect(Collectors.toList());

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(reworkTaskDetails.get(0).getOrganizeCode());
        DdlPreparedPack.getInPackList(ids, "id", packBean);

        return mesPartInspectionDetailRepository.findByHqlWhere(packBean);
    }

    private MesPartTypePicture getMesPartTypePicture(MesReworkTask mesReworkTask) {
        DdlPackBean partBean = DdlPackBean.getDdlPackBean(mesReworkTask.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesReworkTask.getPartNo(), "partNo", partBean);
        MesPart mesPart = mesPartRepository.getByProperty(partBean);
        if (mesPart == null) {
            MesPcnException.throwFlowException(String.format("零件信息不存在,根据物料号查询[%s]", mesReworkTask.getPartNo()));
        }

        DdlPackBean pictureBean = DdlPackBean.getDdlPackBean(mesReworkTask.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesPart.getPartTypeCode(), "partTypeCode", pictureBean);
        MesPartTypePicture mesPartTypePicture = mesPartTypePictureRepository.getByProperty(pictureBean);
        if (mesPartTypePicture == null) {
            throw new ImppBusiException(String.format("图片信息不存在,根据物料号查询[%s]", mesReworkTask.getPartNo()));
        }
        return mesPartTypePicture;
    }

    private MesReworkTask getMesReworkTask(MesReworkTaskRequestModel requestModel) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(requestModel.getOrganizeCode());
        if (!StringUtils.isEmpty(requestModel.getSn())) {
            DdlPreparedPack.getStringEqualPack(requestModel.getSn(), "sn", packBean);
        }
        if (!StringUtils.isEmpty(requestModel.getReworkOrder())) {
            DdlPreparedPack.getStringEqualPack(requestModel.getReworkOrder(), "reworkOrder", packBean);
        }
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.REWORK_TASK_STATUS.CREATE.getValue(), "status", packBean);

        MesReworkTask res = mesReworkTaskRepository.getByProperty(packBean);
        if (res == null) {
            throw new ImppBusiException("返工单信息不存在");
        }
        return res;
    }
    private List<MesReworkTaskDetail> getMesReworkTaskDetail(MesReworkTask mesReworkTask) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(mesReworkTask.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(mesReworkTask.getId(), "pid", packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.REWORK_TASK_DETAIL_STATUS.REWORK_TASK_DETAIL_STATUS_10.getValue(), "status", packBean);

        List<MesReworkTaskDetail> res = mesReworkTaskDetailRepository.findByHqlWhere(packBean);

        return res;
    }

    private void updateMesProduceSn(MesReworkTaskRequestModel requestModel) {
        MesProduceSn produceSn = checkProduceSn(requestModel.getSn(), requestModel.getOrganizeCode());
        produceSn.setQcStatus(MesExtEnumUtil.PRODUCE_QC_STATUS.SCRAP.getValue());
        mesProduceSnRepository.update(produceSn);
    }

    private MesProduceSn checkProduceSn(String serialNumber, String org) {
        //根据扫描的条码查询条码是否存在
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(org);
        DdlPreparedPack.getStringEqualPack(serialNumber, "productSn", packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);
        MesProduceSn produceSn = mesProduceSnRepository.getByProperty(packBean);

        //若不存在则提示条码不存在
        if (StringUtils.isEmpty(produceSn)) {
            throw new ImppBusiException(String.format("【%s】此条码不存在，请检查数据", serialNumber));
        }

        return produceSn;
    }
}
