package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.rulematch;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesEquipVariableCfgRuleMatchDispatchService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesEquipVariableCfgCollectContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesEquipVariableCollectContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionProcessContext;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 取一组全部匹配的结果
 */
@Slf4j
@Service
public class MesEvcRuleMatchCompareValueService implements IMesEquipVariableCfgRuleMatchDispatchService {

    @Override
    public Boolean matchEquipVariableCfgCollectContext(StationRequestBean reqBean, MesProductionProcessContext productionProcessContext, String variableCategory, List<MesEquipVariableCfgCollectContext> collectContextList) {

        Map<String, List<MesEquipVariableCfgCollectContext>> ruleCategoryMap = collectContextList.stream().filter(o -> null != o).collect(Collectors.groupingBy(MesEquipVariableCfgCollectContext::getRuleCategory));

        for(Map.Entry<String, List<MesEquipVariableCfgCollectContext>> entry : ruleCategoryMap.entrySet()) {

            if (null == entry) continue;

            //任意一组规则全部匹配
            if (matchEquipVariableCfgCollectContext(entry.getKey(), entry.getValue())) return true;

        }

        return false;

    }

    private Boolean matchEquipVariableCfgCollectContext(String ruleCategory, List<MesEquipVariableCfgCollectContext> collectContextList) {

        for (MesEquipVariableCfgCollectContext collectContext : collectContextList) {

            if (null == collectContext) continue;

            //任意一个规则不匹配
            if (!matchEquipVariableCfgCollectContext(collectContext)) return false;

        }

        return true;
    }

    private Boolean matchEquipVariableCfgCollectContext(MesEquipVariableCfgCollectContext collectContext) {

        if (StringUtils.isEmpty(collectContext.getValue()) || CollectionUtils.isEmpty(collectContext.getEquipVariableCollectContextList())) return false;

        //一个规则如果对应多个数据变量,则默认必须全部匹配
        for (MesEquipVariableCollectContext equipVariableCollectContext : collectContext.getEquipVariableCollectContextList()) {

            if (null ==  equipVariableCollectContext) continue;

            if (StringUtils.isEmpty(equipVariableCollectContext.getEquipVariableValue()))  return false;

            //比对值是否相等
            if (!compareTo(collectContext.getValue(), equipVariableCollectContext.getEquipVariableValue())) return false;

        }

        return true;
    }

    private Boolean compareTo(String value1, String value2) {
        return value1.toLowerCase().equals(value2.toLowerCase()) ? true : false;
    }

}
