package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesWorkOrderExtService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProdRuleContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionPartContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionProcessContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionPsOutContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.MathOperation;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkCenter;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrder;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.repository.MesWorkOrderRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Description : 保存加工单信息工步
 * @Reference :
 * @Author : wangjie
 * @CreateDate : 2024/6/11 18:50
 * @Modify:
 **/
@Slf4j
@Service("mesWorkOrderSaveStepService")
public class MesWorkOrderSaveStepService extends BaseStepService {

    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;

    @Autowired
    private IMesProductionDispatchContextStepService productionDispatchContextStepService;

    @Autowired
    private IMesWorkOrderExtService workOrderExtService;

    @Autowired
    private MesWorkOrderRepository workOrderRepository;

    @Autowired
    private EntityManager entityManager;

    @Override
    public StepResult execute(StationRequestBean reqBean) {

        StationResultBean resultBean = new StationResultBean();

        StepResult stepResult = StepResult.getSuccessComplete();

        //获取上下文信息
        MesProductionProcessContext productionProcessContext = productionProcessContextStepService.doHandleCurCellEquipment(reqBean);

        //配置错误 抛出异常
        if (!productionProcessContext.getSuccess()) execExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), productionProcessContext.getMessage());

        //存储生产过程上下文对象
        productionProcessContextStepService.saveProductionProcessContext(reqBean, productionProcessContext);

        //获取上下文产品加工规则数据信息集合
        List<MesProdRuleContext> prodRuleContextList = productionDispatchContextStepService.getProdRuleDataContext(reqBean);

        if (CollectionUtils.isEmpty(prodRuleContextList)) execExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), "当前不存在加工规则信息,请重置工序解决!");

        //获取上下文产出零件数据信息集合
        List<MesProductionPartContext> productionPartContextList = productionDispatchContextStepService.getProductionPartContext(reqBean);

        //没有产出零件或者没有工单
        if (CollectionUtils.isEmpty(productionPartContextList) || StringUtils.isEmpty(
                productionPartContextList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getForeignKey()))).findFirst().get().getWorkOrderNo())) return stepResult;

        //获取上下文产出零件数据信息集合
        List<MesProductionPsOutContext> productionPsOutContextList = productionDispatchContextStepService.getProductionPsOutContext(reqBean);

        //从上下文中取出生产线对象
        MesWorkCenter workCenter = productionProcessContext.getWorkCenter();

        //保存生产工单数据
        doHandleWorkOrderData(reqBean, resultBean, productionPartContextList, prodRuleContextList, productionPsOutContextList, workCenter);

        return execSuccessCompleteAndSendMsgReturn(reqBean, resultBean.writeDbLog(MesPcnEnumUtil.WORK_CELL_SCAN_MONITOR_LOG_TYPE.PROCESS.getValue()), stepResult, "保存加工单信息成功!");

    }

    //保存生产工单数据
    private void doHandleWorkOrderData(StationRequestBean reqBean, StationResultBean resultBean, List<MesProductionPartContext> productionPartContextList, List<MesProdRuleContext> prodRuleContextList, List<MesProductionPsOutContext> productionPsOutContextList, MesWorkCenter workCenter) {

        //产出零件信息根据生产工单分组
        Map<String, List<MesProductionPartContext>> ppMap = productionPartContextList.stream().filter(Objects::nonNull).collect(Collectors.groupingBy(MesProductionPartContext::getWorkOrderNo));

        //产出条码 根据生产工单分组
        Map<String, List<MesProductionPsOutContext>> proMap = productionPsOutContextList.stream().filter(Objects::nonNull).collect(Collectors.groupingBy(MesProductionPsOutContext::getWorkOrderNo));
        //产品加工规则 根据foreignKey分组
        Map<Integer, MesProdRuleContext> prMap = prodRuleContextList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getForeignKey()))).collect(Collectors.toMap(MesProdRuleContext::getForeignKey, o -> o));
        //遍历产出零件信息
        for(Map.Entry<String, List<MesProductionPartContext>> entry : ppMap.entrySet()) {

            if (null == entry) continue;
            //获取对应的加工规则
            List<MesProductionPsOutContext> produceSnContextList = proMap.get(entry.getKey());
            //生产数量
            Integer calcCompleteQty = 0;
            //遍历产出物料
            for (MesProductionPsOutContext productionPsOutContext : produceSnContextList) {
                if (null == productionPsOutContext) continue;
                //产出零件是否是
                if (productionPsOutContext.getIsCalcCompleteQty() == CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue()) calcCompleteQty += 1;
            }

            if (calcCompleteQty.compareTo(MesPcnExtConstWords.ZERO) == 0 ) continue;

            MesWorkOrder workOrder = workOrderExtService.getWorkOrder(reqBean.getOrganizeCode(), entry.getValue().get(0).getId());
            entityManager.detach(workOrder);
            if (null == workOrder) execExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), String.format("请检查工单信息,加工单[%s]信息不存在!", entry.getKey()));
            //工单状态分 排序工单 和非排序工单; 现在必须是非排序工单
            if (!MesExtEnumUtil.ORDER_STATUS.checkAllowStatus(workCenter.getCenterType(), workOrder.getWorkOrderStatus()))
                execExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), String.format("请检查工单信息,加工单[%s]信息工单状态[%s]!", entry.getKey(), MesExtEnumUtil.ORDER_STATUS.valueOfDescription(workOrder.getWorkOrderStatus())));
            //工单产线要和当前产线一致
            if (!workOrder.getWorkCenterCode().equals(reqBean.getWorkCenterCode()))
                execExpSendMsgAndThrowEx(reqBean, resultBean.writeDbLog(), String.format("请检查工单信息,加工单[%s]信息所属生产线[%s]与当前生产线[%s]不一致!", entry.getKey(), workOrder.getWorkCenterCode(), reqBean.getWorkCenterCode()));

            List<String> propertyNameList = new ArrayList<>();
            List<Object> propertyValueList = new ArrayList<>();
            //修改完成数量
            if (calcCompleteQty.compareTo(MesPcnExtConstWords.ZERO) > 0) {
                workOrder.setCompleteQty(MathOperation.add(workOrder.getCompleteQty(), calcCompleteQty));
                propertyNameList.add("completeQty");
                propertyValueList.add(workOrder.getCompleteQty());

            }
            //修改未完成数量
            if (calcCompleteQty.compareTo(MesPcnExtConstWords.ZERO) > 0 && workOrder.getUnCompleteQty().compareTo(new Double(0)) > 0 &&
                    MathOperation.compareTo(MathOperation.sub(workOrder.getUnCompleteQty(), new Double(calcCompleteQty)), new Double(0)) >= 0) {
                workOrder.setUnCompleteQty(MathOperation.sub(workOrder.getUnCompleteQty(), new Double(calcCompleteQty)));
                propertyNameList.add("unCompleteQty");
                propertyValueList.add(workOrder.getUnCompleteQty());
            }

            /**
             * 可疑品、报废数量、上下线时间；
             * sql改成修改固定字段
             * 在上下文中获取busi中获取prodResult上下文  是数字  （标志着条码质量状态，与条码质量状态枚举一致）
             * 上下文中 已经按照工单分组了，因为有一模多腔问题
             */

            String prodResult = productionDispatchContextStepService.getProductResultContext(reqBean);
            if (!StringUtils.isEmpty(prodResult)) {
                //可疑品
                boolean isSuspicious = MesExtEnumUtil.PRODUCE_QC_STATUS.SUSPICIOUS.getValue() == Integer.parseInt(prodResult);
                if (isSuspicious && !productionPsOutContextList.isEmpty()){
                    int suspiciousQty = productionPsOutContextList.size();
                    workOrder.setSuspiciousPartQty(MathOperation.sub(workOrder.getSuspiciousPartQty(), new Double(suspiciousQty)));
                    propertyNameList.add("suspiciousPartQty");
                    propertyValueList.add(workOrder.getSuspiciousPartQty());
                }
                //报废数量
                boolean isScrap = MesExtEnumUtil.PRODUCE_QC_STATUS.SCRAP.getValue() == Integer.parseInt(prodResult);
                if (isScrap && !productionPsOutContextList.isEmpty()){
                    int scrapQty = productionPsOutContextList.size();
                    workOrder.setScrapPartQty(MathOperation.sub(workOrder.getScrapPartQty(), new Double(scrapQty)));
                    propertyNameList.add("scrapPartQty");
                    propertyValueList.add(workOrder.getScrapPartQty());
                }
            }

            if (MathOperation.compareTo(workOrder.getCompleteQty(), workOrder.getQty()) >= 0 && workOrder.getWorkOrderStatus().compareTo(MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue()) != 0) {
                workOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.COMPLETE.getValue());
                propertyNameList.add("workOrderStatus");
                propertyValueList.add(workOrder.getWorkOrderStatus());
            } else if (workOrder.getCompleteQty() > 0 && workOrder.getWorkOrderStatus() == MesExtEnumUtil.ORDER_STATUS.RELEASE.getValue()) {
                workOrder.setWorkOrderStatus(MesExtEnumUtil.ORDER_STATUS.PROCESS.getValue());
                propertyNameList.add("workOrderStatus");
                propertyValueList.add(workOrder.getWorkOrderStatus());
            }
            //如果数量都没有修改,不操作数据库
            if (!propertyNameList.isEmpty()) {
                ConvertBean.serviceModelUpdate(workOrder, reqBean.getUserInfo());
                workOrderExtService.update(workOrder);
               /* propertyNameList.add("modifyUser");
                propertyValueList.add(reqBean.getUserInfo());
                propertyNameList.add("modifyDatetime");
                propertyValueList.add(TimeTool.getNowTime(true));
                DdlPackBean workOrderPackBean = DdlPackBean.getDdlPackBean(reqBean.getOrganizeCode());
                DdlPreparedPack.getNumEqualPack(workOrder.getId(),"id",workOrderPackBean);
                workOrderRepository.updateByProperties(propertyNameList.toArray(new String[]{}), propertyValueList.toArray(), workOrderPackBean);
*/            }
        }
    }

}
