package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step.context;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionCustomContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesCellEquipContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProdShiftContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionAssemblyNosortContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.mes.pcn.util.StationKvBeanUtil;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.mes.model.StationKvBean;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import com.alibaba.fastjson.JSONObject;
import com.google.common.base.Objects;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.StringJoiner;
import java.util.concurrent.atomic.AtomicReference;

/**
 * @Description : 获取生产过程上下文对象接口实现【BUSI】
 * @Author : wangjie
 **/
@Slf4j
@Service("mesProductionCustomContextStepService")
public class MesProductionCustomContextStepService extends BaseStepService implements IMesProductionCustomContextStepService {

    //发送设备质量信息
    @Override
    public void sendEquipQualityMessage(StationRequestBean reqBean, MesCellEquipContext cellEquipContext) {
        StationResultBean resultBean = new StationResultBean()
                .busiType(MesPcnEnumUtil.STATION_BUSI_TYPE.QUALITY_MODULE.getValue()).dataType(MesPcnEnumUtil.STATION_DATA_TYPE.COLOR.getValue()).resultObj(MesExtEnumUtil.EQUIP_LOG_QUALITY.valueOfColor(cellEquipContext.getQuality()));
        this.sendMessage(reqBean, resultBean);
    }

    //工位加工锁KEY
    private String getProductionLockContextKey(StationRequestBean reqBean) { return getFsmBusikey(reqBean, new StringJoiner(MesPcnExtConstWords.COLON).add(MesPcnExtConstWords.PRODUCTION_PROCESS_CONTEXT).add(MesPcnExtConstWords.PRODUCTION_LOCK_CONTEXT).toString()); }

    //验证上下文工位加工锁是否存在
    @Override
    public Boolean checkProductionLockIsExistContext(StationRequestBean reqBean) {
        String value = getFsmBusiData(reqBean.getOrganizeCode(), getProductionLockContextKey(reqBean));
        return !StringUtils.isEmpty(value) ? true : false;
    }

    //保存上下文加工锁
    @Override
    public Boolean saveProductionLockContext(StationRequestBean reqBean, String value) {
        return saveFsmBusiData(reqBean.getOrganizeCode(), getProductionLockContextKey(reqBean), value);
    }

    //删除上下文加工锁
    @Override
    public void deleteProductionLockContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getProductionLockContextKey(reqBean));
    }

    //上个扫描/读取信息KEY
    private String getLastScanContextKey(StationRequestBean reqBean) { return getFsmBusikey(reqBean, new StringJoiner(MesPcnExtConstWords.COLON).add(MesPcnExtConstWords.PRODUCTION_PROCESS_CONTEXT).add(MesPcnExtConstWords.LAST_SCAN_INFO_CONTEXT).toString()); }

    //存储上个扫描/读取信息
    @Override
    public Boolean saveLastScanContext(StationRequestBean reqBean, List<StationKvBean> dataList) {
        return saveFsmBusiData(reqBean.getOrganizeCode(), getLastScanContextKey(reqBean), JSONObject.toJSONString(dataList));
    }

    //获取上个扫描/读取信息
    @Override
    public List<StationKvBean> getLastScanContext(StationRequestBean reqBean) {
        String lastScanContent = getFsmBusiData(reqBean.getOrganizeCode(), getLastScanContextKey(reqBean));
        return StringUtils.isEmpty(lastScanContent) ? new ArrayList<>() : JSONObject.parseArray(lastScanContent, StationKvBean.class);
    }

    //发送工步内容
    @Override
    public void doSendStepContextMessage(StationRequestBean reqBean) {
        sendStepContextMessage(reqBean, getLastScanContext(reqBean));
    }

    //发送工步内容
    @Override
    public void doSendStepContextMessage(StationRequestBean reqBean, String scanInfo, MesExtEnumUtil.CELL_MESSAGE_SOURCE source) {

        List<StationKvBean> lastScanContext = getLastScanContext(reqBean);

        lastScanContext = StationKvBeanUtil.addStationKvBeanLimix100ThenHalf(lastScanContext, new StationKvBean().name(source.getDescription()).value(scanInfo));

        saveLastScanContext(reqBean, lastScanContext);

        sendStepContextMessage(reqBean, lastScanContext);

    }

    //发送工步内容
    @Override
    public void sendStepContextMessage(StationRequestBean reqBean, List<StationKvBean> lastScanContext) {
        StationResultBean resultBean = new StationResultBean()
                .busiType(MesPcnEnumUtil.STATION_BUSI_TYPE.STEP_CONTENT.getValue()).dataType(MesPcnEnumUtil.STATION_DATA_TYPE.KEY_VALUE.getValue())
                .resultList(StationKvBeanUtil.addStationKvBeanList(new ArrayList<>(),
                        lastScanContext.size() > 1 ? lastScanContext.get(1).name("上次" + lastScanContext.get(1).getName()).key(MesPcnExtConstWords.LAST_SCAN_INFO) : null,
                        lastScanContext.size() > 0 ? lastScanContext.get(0).name("当前" + lastScanContext.get(0).getName()).key(MesPcnExtConstWords.SCAN_INFO) : null));
        this.sendMessage(reqBean, resultBean);
    }

    //获取生产线的当前班组班次信息
    @Override
    public MesProdShiftContext getMesProdShiftKvBean(String orgainzeCode, String workCenterCode) {
        List<StationKvBean> stationKvBeans = getProdShiftDataContext(orgainzeCode, workCenterCode);

        MesProdShiftContext mesProdShiftContext = new MesProdShiftContext();
        if (CollectionUtils.isEmpty(stationKvBeans)) {
            return  null;
        }
        StationKvBean shiftGroupKvBean = stationKvBeans.stream().filter(stationKvBean -> Objects.equal(stationKvBean.getKey(), "shiftGroup")).findFirst().orElse(null);
        StationKvBean shiftCodeKvBean = stationKvBeans.stream().filter(stationKvBean -> Objects.equal(stationKvBean.getKey(), "shiftCode")).findFirst().orElse(null);

        mesProdShiftContext.setShiftCode(shiftGroupKvBean == null ? null : shiftCodeKvBean.getValue());
        mesProdShiftContext.setShiftGroup(shiftCodeKvBean == null ? null : shiftGroupKvBean.getValue());
        return  mesProdShiftContext;
    }

    //开班信息KEY 【PS:生产线级别】
    private String getProdShiftContextKey(String orgainzeCode, String workCenterCode) { return new StringJoiner(MesPcnExtConstWords.COLON).add(orgainzeCode).add(MesPcnExtConstWords.PROD_SHIFT_DATA_CONTEXT).add(workCenterCode).toString(); }

    //获取上下文班次班组信息
    @Override
    public List<StationKvBean> getProdShiftDataContext(String orgainzeCode, String workCenterCode) {
        String prodShiftJson = getFsmBusiData(orgainzeCode, getProdShiftContextKey(orgainzeCode, workCenterCode));
        return !StringUtils.isEmpty(prodShiftJson) ? JSONObject.parseArray(prodShiftJson, StationKvBean.class) : null;
    }

    //验证上下文班次班组信息是否存在
    @Override
    public Boolean checkProdShiftDataIsExistContext(String orgainzeCode, String workCenterCode) {
        String prodShiftJson = getFsmBusiData(orgainzeCode, getProdShiftContextKey(orgainzeCode, workCenterCode));
        return !StringUtils.isEmpty(prodShiftJson) ? true : false;
    }

    //保存上下文班次班组信息  [JSON]List<StationKvBean>
    @Override
    public Boolean saveProdShiftDataContext(String organizeCode, String workCenterCode, List<StationKvBean> prodShiftDataList) {
        if (CollectionUtils.isEmpty(prodShiftDataList)) return false;
        return saveFsmBusiData(organizeCode, getProdShiftContextKey(organizeCode, workCenterCode), JSONObject.toJSONString(prodShiftDataList));
    }

    //删除上下文班次班组信息
    @Override
    public void deleteProdShiftDataContext(String orgainzeCode, String workCenterCode) {
        deleteFsmBusiData(orgainzeCode, getProdShiftContextKey(orgainzeCode, workCenterCode));
    }

    //加工次数key
    private String getProductionStatisticsContextKey(StationRequestBean reqBean) { return getFsmBusikey(reqBean, new StringJoiner(MesPcnExtConstWords.COLON).add(MesPcnExtConstWords.PRODUCTION_PROCESS_CONTEXT).add(MesPcnExtConstWords.PRODUCTION_STATISTICS_CONTEXT).toString()); }

    //获取上下文工位加工数统计信息
    @Override
    public List<StationKvBean> getProductionStatisticsContext(StationRequestBean reqBean) {
        String productionStatisticsJson = getFsmBusiData(reqBean.getOrganizeCode(), getProductionStatisticsContextKey(reqBean));
        return !StringUtils.isEmpty(productionStatisticsJson) ? JSONObject.parseArray(productionStatisticsJson, StationKvBean.class) : null;
    }

    //保存上下文工位加工数统计信息  [JSON]List<StationKvBean>
    @Override
    public Boolean saveProductionStatisticsContext(StationRequestBean reqBean, List<StationKvBean> productionStatisticsList) {
        if (CollectionUtils.isEmpty(productionStatisticsList)) return false;
        return saveFsmBusiData(reqBean.getOrganizeCode(), getProductionStatisticsContextKey(reqBean), JSONObject.toJSONString(productionStatisticsList));
    }

    //删除上下文工位加工数统计信息
    @Override
    public void deleteProductionStatisticsContext(StationRequestBean reqBean) {
        deleteFsmBusiData(reqBean.getOrganizeCode(), getProductionStatisticsContextKey(reqBean));
    }

    @Override
    public Integer addProductionStatisticsContext(StationRequestBean reqBean, int count) {

        MesProdShiftContext mesProdShiftKvBean = getMesProdShiftKvBean(reqBean.getOrganizeCode(), reqBean.getWorkCenterCode());

        List<StationKvBean> stationKvBeans = getProductionStatisticsContext(reqBean);

        AtomicReference<Integer> amount = new AtomicReference<>();
        amount.set(count);

        if (CollectionUtils.isEmpty(stationKvBeans)) {
            String name = "加工数";
            String value = count + "";
            stationKvBeans = StationKvBeanUtil.addStationKvBeanList(new ArrayList<>(),
                    new StationKvBean(mesProdShiftKvBean.getShiftGroup(), name, value), new StationKvBean("color", "颜色", MesExtEnumUtil.COLOR.BLACK.getValue()));
        } else {
            stationKvBeans.stream().forEach(stationKvBean -> {
                if (stationKvBean.getKey().contains(mesProdShiftKvBean.getShiftGroup())) {
                    stationKvBean.setValue(stationKvBean == null ? count + "" : (Integer.valueOf(stationKvBean.getValue()) + count) + "");
                    amount.set(Integer.valueOf(stationKvBean.getValue()));
                }
            });
        }

        saveProductionStatisticsContext(reqBean, stationKvBeans);

        return amount.get();
    }

    //可复用条码KEY
    private String getRepeatAssemblySnContextKey(String organizeCode, String workCenterCode, String workCellCode) {
        return new StringJoiner(":").add(organizeCode).add(workCenterCode).add(workCellCode).add(MesPcnExtConstWords.PRODUCTION_PROCESS_CONTEXT).add(MesPcnExtConstWords.REPEAT_ASSEMBLY_SN_CONTEXT).toString();
    }

    //根据装配件规则ID获取可复用条码
    @Override
    public String getRepeatAssemblySn(String organizeCode, String workCenterCode, String workCellCode, String assemblyNosortCfgId) {
        String repeatAssemblySnJson = getFsmBusiData(organizeCode, getRepeatAssemblySnContextKey(organizeCode, workCenterCode, workCellCode), assemblyNosortCfgId);
        MesProductionAssemblyNosortContext productionAssemblyNosortContext = !StringUtils.isEmpty(repeatAssemblySnJson) ? JSONObject.parseObject(repeatAssemblySnJson, MesProductionAssemblyNosortContext.class) : null;
        return null != productionAssemblyNosortContext ? productionAssemblyNosortContext.getAssemblySn() : null;
    }

    //根据设备代码获取可复用条码的个数
    @Override
    public Integer getRepeatAssemblySnCount(String organizeCode, String workCenterCode, String workCellCode, String equipmentCode) {
        String countStr = getFsmBusiData(organizeCode, getRepeatAssemblySnContextKey(organizeCode, workCenterCode, workCellCode), equipmentCode);
        return !StringUtils.isEmpty(countStr) ? Integer.valueOf(countStr) : MesPcnExtConstWords.ZERO;
    }

    //根据装配件规则ID获取可复用条码
    @Override
    public Boolean saveRepeatAssemblySn(String organizeCode, String workCenterCode, String workCellCode, MesProductionAssemblyNosortContext productionAssemblyNosortContext) {
        Integer count = getRepeatAssemblySnCount(organizeCode, workCenterCode, workCellCode, productionAssemblyNosortContext.getEquipmentCode()) + 1;
        saveFsmBusiData(organizeCode, getRepeatAssemblySnContextKey(organizeCode, workCenterCode, workCellCode), productionAssemblyNosortContext.getEquipmentCode(), count.toString());
        return saveFsmBusiData(organizeCode, getRepeatAssemblySnContextKey(organizeCode, workCenterCode, workCellCode), productionAssemblyNosortContext.getSourceId().toString(), JSONObject.toJSONString(productionAssemblyNosortContext));
    }

    //删除上下文可复用条码信息
    @Override
    public void deleteRepeatAssemblySn(String organizeCode, String workCenterCode, String workCellCode) {
        deleteFsmBusiData(organizeCode, getRepeatAssemblySnContextKey(organizeCode, workCenterCode, workCellCode));
    }

}
