package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step.context;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionCustomContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionPsOutContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.actor.shipping.dispatch.IFsmCommonService;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.process.BaseProcessMonitorService;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.swslog.ISwsWriteDbLogService;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Primary;
import org.springframework.stereotype.Service;

@Service
@Primary
public class MesProductionProcessMonitorService extends BaseProcessMonitorService {

    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;

    @Autowired
    private IMesProductionDispatchContextStepService productionDispatchContextStepService;

    @Autowired
    private IMesProductionCustomContextStepService productionCustomContextStepService;

    @Autowired
    private IFsmCommonService fsmCommonService;

    @Autowired
    private ISwsWriteDbLogService writeDbLogService;

    //工序开始
    @Override
    public Boolean doProcessStart(StationRequestBean requestBean) {
        //保存上下文工位扫描监控信息
        productionProcessContextStepService.saveScanMonitorContext(requestBean);
        //清除工步客制化区域数据【装配件清单】
        this.sendMessage(requestBean, new StationResultBean().busiType(MesPcnEnumUtil.STATION_BUSI_TYPE.STEP_CUSTOM_CONTENT.getValue()).dataType(MesPcnEnumUtil.STATION_DATA_TYPE.TABLES.getValue()));
        return true;
    }

    //工序结束
    @Override
    public Boolean doProcessComplete(StationRequestBean requestBean) {
        //补DB日志 [获取开模ID赋值scanInfo] [获取上下文产出条码数据信息集合赋值resultList]
        writeDbLogService.doRestoreDbLog(requestBean, new StationResultBean().restoreDbLog().
                scanInfo(productionProcessContextStepService.getScanMonitorContextMouldRecordId(requestBean)).resultList(productionDispatchContextStepService.getProductionPsOutContext(requestBean)));
        //清除上下文中的所有业务数据
        productionDispatchContextStepService.doFlushProductionDispatchContext(requestBean);
        //发送工序完成音
        this.sendMessage(requestBean, new StationResultBean().resultObj(MesPcnExtConstWords.PROCESS_COMPLETE).busiType(MesPcnEnumUtil.STATION_BUSI_TYPE.CUSTOM_COMPONENT.getValue()).dataType(MesPcnEnumUtil.STATION_DATA_TYPE.SOUND.getValue()));
        return true;
    }

    //重置工序
    @Override
    public Boolean doJumpProcess(StationRequestBean requestBean) {
        //清除上下文中的所有业务数据
        productionDispatchContextStepService.doFlushProductionDispatchContext(requestBean);
        return true;
    }

    //客户端下线
    @Override
    public Boolean doOffLine(StationRequestBean requestBean) {
        //删除当前工步集
        fsmCommonService.doFlushCache(requestBean);
        //清除上下文中的所有业务数据
        productionDispatchContextStepService.doFlushProductionDispatchContext(requestBean);
        //清除上下文中的所有基础数据
        productionProcessContextStepService.doFlushProductionProcessContext(requestBean);
        return true;
    }

    //工位流程重置 【父类已经调用: 删除当前工步集; 清除工位工序流程; TASK_COMPLETE】
    @Override
    public Boolean fsmReset(StationRequestBean requestBean) {
        //清除上下文中的所有业务数据
        productionDispatchContextStepService.doFlushProductionDispatchContext(requestBean);
        //清除上下文中的所有基础数据
        productionProcessContextStepService.doFlushProductionProcessContext(requestBean);
        return true;
    }

}
